require_relative '../../features_helper'

describe 'application generated with shoestrap new <application_name>' do
  before :all do
    generate_test_app_with_shoestrap
  end

  let (:gemfile_content) { application_file('Gemfile') }
  let (:application_rb_content) { application_file('config/application.rb') }
  let (:application_layout) { application_file('app/views/layouts/application.html.haml') }
  let (:application_header) { application_file('app/views/partials/_header.html.haml') }
  let (:application_footer) { application_file('app/views/partials/_footer.html.haml') }
  let (:application_layout_typekit) { application_file('app/views/layouts/_typekit.html.haml') }
  let (:application_helper) { application_file('app/helpers/application_helper.rb') }
  let (:development_rb_content) { application_file('config/environments/development.rb') }

  it 'has timezone set for switzerland' do
    expect(application_rb_content).to match(/config.time_zone = 'Bern'/)
  end

  it 'has :de as default locale' do
    expect(application_rb_content).to match(/config.i18n.default_locale = :de/)
  end

  it 'has :de as only available locale for a satrt' do
    expect(application_rb_content).to match(/config.i18n.available_locales = \[:de\]/)
  end

  it 'has translation files load path' do
    expect(application_rb_content).to match(/config.i18n.load_path \+= Dir\[Rails.root.join\('config', 'locales', '\*\*', '\*.{rb,yml}'\).to_s\]/)
  end

  it 'has the shoestrap version in test set to the current path in Gemfile' do
    expect(gemfile_content.split("\n")).to include("gem \"shoestrap\", path: \"#{Dir.pwd + '/features/..'}\"")
  end

  it 'has the typekit key set to false' do
    expect(application_rb_content).to match(/config.typekit_key = false/)
  end

  # TODO: fix this in rails. Gems added with gem method of app template have double quotes!

  %w[
     simple_form
     haml
     haml-rails
     simple-navigation
     mail
     fabrication
     kuhsaft
     selectivizr-rails
     modernizr-rails
     bourbon
     therubyracer
     shoestrap
     inherited_resources
  ].each do |gem_name|
    it "has #{gem_name} in the Gemfile" do
      expect(gemfile_content).to include_gem(gem_name)
    end
  end

  %w[
    pry-rails
    database_cleaner
    guard-rspec
    guard-spring
    rspec-rails
    mailcatcher
  ].each do |gem_name|
    it "has #{gem_name} in the Gemfile" do
      expect(gemfile_content).to include_gem(gem_name)
    end
  end

  it 'has the pg gem in the Gemfile' do
    expect(gemfile_content).to include_gem('pg')
  end

  context 'application layout' do
    it 'has no application layout written in erb' do
      expect { application_file('app/views/layouts/application.html.erb') }.to raise_error(Errno::ENOENT)
    end

    it 'has an application layout written in haml' do
      expect(application_layout).not_to be_nil
    end

    it 'sets the html lang' do
      expect(application_layout).to match "%html.no-js{ lang: I18n.locale }"
    end

    it 'renders content_for :head if it is defined' do
      expect(application_layout).to match(/content_for\?\(:head\)\n      = yield\(:head\)\n    - else\n      %title 'Untitled'\n/)
    end

    it 'has meta tags' do
      expect(application_layout).to match "%meta{ charset: 'utf-8' }"
      expect(application_layout).to match "%meta{ 'http-equiv' => 'X-UA-Compatible', content: 'IE=edge' }"
      expect(application_layout).to match "%meta{ name: 'keywords', content: '' }"
      expect(application_layout).to match "%meta{ name: 'description', content: '' }"
      expect(application_layout).to match "%meta{ name: 'viewport', content: 'width=device-width, initial-scale=1.0' }"
      expect(application_layout).to match "= csrf_meta_tags"
    end

    it 'has favicons' do
      expect(application_layout).to match(/favicon_link_tag\('favicons\/129x129\.png',\ rel:\ 'apple\-touch\-icon',\ type:\ 'image\/png'\)/)
      expect(application_layout).to match(/favicon_link_tag\('favicons\/32x32\.png',\ rel:\ 'icon',\ type:\ 'image\/png'\)/)
      expect(application_layout).to match(/favicon_link_tag\('favicons\/32x32\.ico'\)/)
    end

    it 'has an application stylehseet link' do
      expect(application_layout).to match "= stylesheet_link_tag    'application', media: 'all', 'data-turbolinks-track' => true"
    end

    it 'has javascript links' do
      expect(application_layout).to match "= javascript_include_tag 'modernizr', 'data-turbolinks-track' => true"
      expect(application_layout).to match "= javascript_include_tag 'application', 'data-turbolinks-track' => true"
      expect(application_layout).to match "= javascript_include_tag 'selectivizr'"
    end

    it 'has a body class helper included' do
      expect(application_layout).to match "%body{ class: body_class }"
    end

    it 'renders the navigation' do
      expect(application_header).to match "= render_navigation"
    end

    it 'renders the content' do
      expect(application_layout).to match "= yield"
    end

    it 'renders the typekit' do
      expect(application_layout).to match "= render 'layouts/typekit'"
    end

    it 'has google analytics integrated' do
      expect(application_layout).to match(/e\.src = \"\/\/www\.google-analytics\.com\/analytics\.js\"/)
    end
  end

  context 'documentation file' do

    it 'contains proper documentation' do
      pending "implement proper readme template"
      expect(application_file('README.md')).not_to match('README would normally document whatever steps are necessary to get the')
    end
  end

  context 'favicons' do
    it 'has a favicon' do
      expect(application_file('app/assets/images/favicons/32x32.png')).not_to be_nil
    end

    it 'has a favicon for iDevices' do
      expect(application_file('app/assets/images/favicons/129x129.png')).not_to be_nil
    end

    it 'has a favicton for IE' do
      expect(application_file('app/assets/images/favicons/32x32.ico')).not_to be_nil
    end
  end

  context 'layout partials' do
    it 'has a typekit partial' do
      expect(application_layout_typekit).not_to be_nil
    end

    context 'typekit partial' do
      it 'checks for typekit_key configuration' do
        expect(application_layout_typekit).to match "- if Rails.application.config.typekit_key.present?"
      end

      it 'has the typekit js integrated' do
        expect(application_layout_typekit).to match(/javascript_include_tag\ "\/\/use\.typekit\.net\/\#\{Rails\.application\.config\.typekit_key\}\.js"/)
        expect(application_layout_typekit).to match(/try\{Typekit\.load\(\);\}catch\(e\)\{\}/)
      end
    end
  end

  context 'application helper' do
    it 'has an application layout written in haml' do
      expect(application_helper).not_to be_nil
    end

    it 'has a body_class method' do
      expect(application_helper).to match "def body_class"
    end
  end

  context 'generators' do
    it 'use haml as a template engine' do
      expect(application_rb_content).to match "g.template_engine :haml"
    end

    it 'use rspec as test framework' do
      expect(application_rb_content).to match "test_framework :rspec"
    end

    it 'do not generate stylesheets' do
      expect(application_rb_content).to match "g.stylesheets false"
    end

    it 'do not generate javascripts' do
      expect(application_rb_content).to match "g.javascripts false"
    end

    it 'do not generate assets' do
      expect(application_rb_content).to match "g.assets false"
    end

    it 'does not generate helpers' do
      expect(application_rb_content).to match "g.helper false"
    end

    context 'mailcatcher' do
      it 'is configured for development environment' do
        expect(development_rb_content).to match(/config\.action_mailer\.delivery_method = :smtp/)
        expect(development_rb_content).to match(/config\.action_mailer\.smtp_settings = { address: 'localhost', port: 1025 }/)
      end

      it 'is documented in the readme' do
        expect(application_file('README.md')).to match(/This application uses Mailcatcher/)
      end
    end
  end
end
