require_relative '../../features_helper'

describe 'rails generate shoestrap:cms' do

  context 'without kuhsaft' do

    options = {kuhsaft: false}
    let(:cars_controller) { application_file('app/controllers/cms/cars_controller.rb', options) }
    let(:app_path){ application_path(options) }

    before :all do
      generate_test_app_with_shoestrap options
    end

    it 'is available in the generated app' do
      cli_output = rails 'generate', options
      expect(cli_output).to match(/shoestrap:cms/)
    end

    context 'generating a cms resource' do
      let(:navigation_config_path) { File.join(app_path, 'config/navigation.rb') }
      let(:navigation_config) { application_file('config/navigation.rb', options) }
      let(:gemfile_content) { application_file('Gemfile', options) }

      before :all do
        rails 'generate shoestrap:cms car tires:integer model:string', options
      end

      context 'for the first time' do
        it 'sets up the cms routes namespace' do
          expect(application_file('config/routes.rb', options)).to match(/namespace :cms do/)
        end

        it 'sets up controller inheritance for the cms backend' do
          expect(application_file('app/controllers/shoestrap/base_controller.rb', options)).to match('ActionController::Base')
          expect(application_file('app/controllers/shoestrap/base_controller.rb', options)).to match('inherit_resources')
        end

        it 'generates inherited resource views' do
          views = Dir[File.join(app_path, 'app/views/shoestrap/base/*.html.haml')]
          views.map!{|v| v.scan(/^.+app\/views\/shoestrap\/base\/(.*).html.haml/).flatten.first.to_sym }
          expected_action_views = [:edit, :index, :new, :"_form"]
          expect(expected_action_views - views).to be_empty
        end
      end

      context 'generates a CRUD controller' do
        it 'inherits from the shoestrap base controller' do
          expect(cars_controller).to match(/Shoestrap..BaseController/)
        end
      end

      it 'generates a model' do
        expect(application_file('app/models/car.rb', options)).not_to be_nil
      end

      it 'uses the fields given in the command line to pre-fill editable and index attributes' do
        expect(application_file('app/models/car.rb', options)).to match('editable_attributes :tires, :model')
        expect(application_file('app/models/car.rb', options)).to match('index_attributes :tires, :model')
      end

      it 'generates a migration' do
        expect(Dir.glob("#{app_path}/db/migrate/*create*cars*").size).to be 1
      end

      it 'adds a resource route' do
        expect(application_file('config/routes.rb', options)).to match(/resources :cars/)
      end

      it 'does not generate routing specs' do
        routing_spec = File.join(app_path, 'spec/routing/cars_routing_spec.rb')
        expect(File.exists? routing_spec).to be_false
      end

      it 'does not generate view specs' do
        view_specs = File.join(app_path, 'spec/views/cars')
        expect(File.exists? view_specs).to be_false
      end

      it 'does not generate controller specs' do
        controller_specs = File.join(app_path, 'spec/controllers/cars_controller_spec.rb')
        expect(File.exists? controller_specs).to be_false
      end

      it 'does not generate json views' do
        expect(Dir.glob("#{app_path}/app/views/cars/*.jbuilder").size).to be 0
      end

      it 'should not contain the kuhsaft gem' do
        expect(gemfile_content).not_to include_gem('kuhsaft')
      end

      it 'should not have the kuhsaft initializer' do
        expect(File.exists?(File.join(app_path, 'config/initializers/kuhsaft.rb'))).to be_false
      end

    end # generating a cms resource
  end # without default cms

  context 'with kuhsaft' do
    options = {kuhsaft: true}
    let(:cars_controller) { application_file('app/controllers/cms/cars_controller.rb', options) }
    let(:gemfile_content) { application_file('Gemfile', options) }
    let(:app_path){ application_path(options) }
    let(:navigation_config_path) { File.join(app_path, 'config/cms_navigation.rb') }
    let(:navigation_config) { application_file('config/cms_navigation.rb', options) }

    before :all do
      generate_test_app_with_shoestrap options
    end

    context 'generating a cms resource' do
      before :all do
        rails 'generate shoestrap:cms car tires:integer model:string', options
      end

      context 'the shoestrap base controller' do
        it 'inherits from the kuhsaft admin controller' do
          expect(application_file('app/controllers/shoestrap/base_controller.rb', options)).to match('Kuhsaft::Cms::AdminController')
        end
      end

      it 'should contain the kuhsaft gem' do
        expect(gemfile_content).to include_gem('kuhsaft')
      end

      it 'should have the kuhsaft initializer' do
        expect(File.exists?(File.join(app_path, 'config/initializers/kuhsaft.rb'))).to be_true
      end

      it 'sets up the cms navigation file' do
        expect(File.exists? navigation_config_path).to be_true
      end

      it 'adds the generated resource to the cms navigation' do
        expect(navigation_config).to match('cms_cars_path')
      end
    end
  end # with kuhsaft
end # rails generate shoestrap:cms
