require_relative '../../lib/shoestrap/application_generator'
require_relative '../spec_helper'

module Shoestrap
  describe ApplicationGenerator do

    let(:default_options) { { css: 'bootstrap', database: 'postgresql', kuhsaft: true } }
    let(:template) { (Pathname.new(__FILE__) + '../../../templates/application_generator/application_template.rb').realpath }

    describe 'rails template' do
      it 'should use current shoestrap version' do
        tp = double(:template_processor)
        tp.should_receive(:remove_file).with('app/views/layouts/application.html.erb')
        tp.should_receive(:require).with(File.expand_path('../../lib/shoestrap/version'))
        tp.should_receive(:inject_into_file)
        tp.stub(:directory)
        tp.stub(:copy_file)
        tp.stub(:application)
        tp.should_receive(:gem_group).twice
        expected_gems = ['shoestrap']
        tp.stub(:gem) {|*args|
          expected_gems.delete(args[0])
          if args[0] == 'shoestrap'
            expect(args[1]).to eq(Shoestrap::VERSION)
          end
        }
        tp.instance_eval template.read
        expect(expected_gems).to eq []
      end
    end

    describe '#start' do
      it 'says it is going to generate an app' do
        ApplicationGenerator.stub(:shoestrap_generators).and_return(:true)
        output = capture(:stdout, :stderr) do
          ApplicationGenerator.start('test_app', default_options)
        end
        expect(output).to match(/\[shoestrap\] Generating new Rails Application test_app/)
      end

      it 'calls rails new with the shoestrap template' do
        ApplicationGenerator.stub(:shoestrap_generators).and_return(:true)
        capture(:stdout, :stderr) do
          expect(Shell).to receive(:exec).with("rails new test_app -m #{File.join(
                                                             File.dirname(__FILE__).gsub('spec','lib'),
                                                             '../../templates/application_generator/application_template.rb')} -d postgresql")
          ApplicationGenerator.start('test_app', default_options)
        end
      end

      it 'runs the shoestrap generators' do
        capture(:stdout, :stderr) do
          expect(ApplicationGenerator).to receive(:shoestrap_generators)
          ApplicationGenerator.start('test_app', default_options)
        end
      end
    end

    describe '#shoestrap_generators' do

      before :each do
        ApplicationGenerator.stub(:call_generator)
        ApplicationGenerator.stub(:install_gems)
        ApplicationGenerator.stub(:apply_special_sauce)
        Dir.stub(:chdir).and_yield
      end

      it 'installs required gems' do
        expect(ApplicationGenerator).to receive(:install_gems)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
      end

      it 'sets up the css framework' do
        expect(ApplicationGenerator).to receive(:call_generator)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
      end

      it 'sets up the smacss files' do
        expect(ApplicationGenerator).to receive(:call_generator)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
      end

      it 'sets up the coffee files' do
        expect(ApplicationGenerator).to receive(:call_generator)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
      end

      it 'sets up the bdd environment' do
        expect(ApplicationGenerator).to receive(:call_generator)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
      end

      it 'sets up kuhsaft' do
        expect(ApplicationGenerator).to receive(:call_generator)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
      end

      it 'applies special sauce when the correct environment variable is set' do
        original_value = ENV['GIVE_ME_SPECIAL_SAUCE']
        ENV['GIVE_ME_SPECIAL_SAUCE'] = "true"
        expect(ApplicationGenerator).to receive(:apply_special_sauce)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
        ENV['GIVE_ME_SPECIAL_SAUCE'] = original_value
      end

      it 'does not apply special sauce normally' do
        original_value = ENV['GIVE_ME_SPECIAL_SAUCE']
        ENV['GIVE_ME_SPECIAL_SAUCE'] = nil
        expect(ApplicationGenerator).not_to receive(:apply_special_sauce)
        ApplicationGenerator.shoestrap_generators('test_app', default_options)
        ENV['GIVE_ME_SPECIAL_SAUCE'] = original_value
      end
    end

    describe '#install_gems' do
      it 'delegates to the Gem Install Generator' do
        Shell.stub(:exec)
        expect(GemInstallGenerator).to receive(:start)
        ApplicationGenerator.install_gems []
      end

      it 'delegates to the Gem Install Generator' do
        GemInstallGenerator.stub(:start)
        expect(Shell).to receive(:exec).with('bundle')
        ApplicationGenerator.install_gems []
      end
    end

    describe '#setup_css_framework' do
      it 'runs the generator it was called with' do
        DummyGenerator = double
        expect(DummyGenerator).to receive('start')
        ApplicationGenerator.call_generator('dummy')
      end
    end

    describe '#setup_smacss_files' do
      it 'runs the smacss files generator' do
        expect(SmacssFilesGenerator).to receive('start')
        ApplicationGenerator.call_generator('smacss_files')
      end
    end

    describe '#setup_coffee_files' do
      it 'runs the coffee files generator' do
        expect(CoffeeFilesGenerator).to receive('start')
        ApplicationGenerator.call_generator('coffee_files')
      end
    end

    describe '#setup_bdd_environment' do
      it 'runs the rspec generator' do
        expect(BddGenerator).to receive('start')
        ApplicationGenerator.call_generator('bdd')
      end
    end

    describe '#setup_kuhsaft' do
      it 'delegates to the Gem Install Generator' do
        expect(KuhsaftGenerator).to receive(:start)
        ApplicationGenerator.call_generator('kuhsaft')
      end
    end

    describe '#apply_special_sauce' do
      it 'calls sauce apply on current directory' do
        expect(Shell).to receive(:exec).with('sauce apply . --appname=foo')
        ApplicationGenerator.apply_special_sauce('foo')
      end
    end

    describe '#required_gems' do
      it 'includes kuhsaft by default' do
        expect(ApplicationGenerator.required_gems(default_options)).to include('kuhsaft')
      end

      it 'includes bootstrap-sass by default' do
        expect(ApplicationGenerator.required_gems(default_options)).to include('bootstrap-sass')
      end

      it 'includes zurb-foundation when css option is set to foundation' do
        expect(ApplicationGenerator.required_gems(default_options.merge( css: 'foundation' ))).to include('zurb-foundation')
      end
    end
  end
end
